@@ -1,6 +1,6 @@
 from re import split, sub
 from typing import DefaultDict, Dict, Union, TextIO, List, Any, Generator, Tuple, Iterable, Mapping, Hashable, Type
-from sys import argv, version_info
+from sys import argv
 from os import getenv, name as os_name, environ as os_environ
 if os_name == 'nt':
     from nt import environ as nt_environ
@@ -37,7 +37,7 @@ def argv_to_dict(args: List[str], aliases: Dict[str, str] = None) -> DefaultDict
     result = defaultdict(list)
     for a in args:
         # arguments are prefixed with -, -- or / - no distinction for long names, so --h or -help would be valid
-        if len(a) > 0 and a[0] in SWITCH_CHARS:
+        if len(a) > 0 and a[0] in SWITCH_CHARS and ' ' not in a:
             if len(a) == 1:
                 raise SyntaxError('Syntax error in argument: {}'.format(a))
             key = a[2:] if a[:2] == '--' else a[1:]
@@ -97,18 +97,26 @@ class DictConfig(dict):
         'request_header': 'rh', 'header': 'rh'
     }
 
+    def _init_attr(self, name, value):
+        """
+        Helper to avoid triggering attribute/key mapping during initialisation
+        :param name: name of the attribute to set
+        :param value: value to set it to
+        :return: None
+        """
+        super(DictConfig, self).__setattr__(name, value)
+
     def __init__(self, *args, no_globals: Union[bool, dict] = False, no_key_error: bool = False,
                  skip_lists: bool = False, no_compound_keys: bool = False, skip_iterables: bool = False):
         """
         Constructor method
         """
         super(DictConfig, self).__init__(*args)
-        self.no_compound_keys = no_compound_keys
-        self._casting = False  # overrides no_compound_keys during casting
-        self.no_key_error = no_key_error
-        if no_globals is None:
-            self.globals = None
-        elif no_globals:
+        self._init_attr('no_compound_keys', no_compound_keys)
+        self._init_attr('_casting', False)  # overrides no_compound_keys during casting
+        self._init_attr('no_key_error', no_key_error)
+        self._init_attr('globals', {})
+        if no_globals:
             if isinstance(no_globals, dict):
                 if isinstance(no_globals, DictConfig):
                     # copy the reference, if no_globals is already a Config
@@ -117,25 +125,26 @@ class DictConfig(dict):
                     # any other dict type will be cast (and thus copied) to type of self, with default settings
                     self.globals = self.__class__(no_globals)
             else:
-                self.globals = None
+                self.globals = {}
         elif GLOBALS_KEY not in self or not isinstance(super(DictConfig, self).__getitem__(GLOBALS_KEY), dict):
             # globals as part of a config only work if they are in the config and are actually a dict (or a Config)
             if GLOBALS_KEY in self:
                 del self[GLOBALS_KEY]
-            self.globals = None
+            self.globals = {}
         else:
             # cast _globals dict in self to self type
             self.globals = self.__class__(super(DictConfig, self).__getitem__(GLOBALS_KEY))
             del self[GLOBALS_KEY]
-        self.disable_globals = False
-        self.filename = None
-        self.arguments = None
-        self.env_var_prefix = None
-        self.cfg_filename = None
-        self.parameters = None
-        self.from_arguments = []
+
+        self._init_attr('disable_globals', False)
+        self._init_attr('file_path', None)
+        self._init_attr('arguments', None)
+        self._init_attr('env_var_prefix', None)
+        self._init_attr('parameters', None)
+        self._init_attr('from_arguments', [])
+
         # this attribute is only for the benefit of the Config subclass, which exposes it as a property
-        self._shadow_attrs = False
+        self._init_attr('_shadow_attrs', False)
         # replace dicts in Config with equivalent Config
         self._dicts_to_config(self, skip_iterables=skip_lists or skip_iterables)
 
@@ -143,6 +152,10 @@ class DictConfig(dict):
     def direct(self):
         return _NoGlobalsProxy(self)
 
+    @property
+    def filename(self):
+        return None if self.file_path is None else str(self.file_path)
+
     def _dict_cast(self, a_dict: dict, from_type: type, to_type: type, skip_iterables: bool = False) -> dict:
         """
         Replace every dictionary of type `from_type` in a_dict with a dictionary of `to_type` configured like self,
@@ -283,7 +296,7 @@ class DictConfig(dict):
         return dict.__getitem__(self, key)
 
     def _subst_globals(self, value: Any) -> Any:
-        if self.globals is None or self.disable_globals:
+        if not self.globals or self.disable_globals:
             return value
         if isinstance(value, str):
             try:
@@ -323,7 +336,7 @@ class DictConfig(dict):
         :raises KeyError: if the key cannot be found (and self.no_key_error is True, None otherwise)
         """
         if self.no_compound_keys:
-            return super(DictConfig, self).__getitem__(key)
+            return self._subst_globals(super(DictConfig, self).__getitem__(key))
 
         keys = self._split_key(key)
 
@@ -347,13 +360,19 @@ class DictConfig(dict):
                     raise KeyError('Multi-part key, but `{}` is not a dictionary or Config.'.format(keys[0]))
             return self._subst_globals(super(DictConfig, self).__getitem__(keys[0]))
 
-    def get(self, key: Hashable, default: Any = None) -> Any:
+    def get(self, key: Hashable, default: Any = None, no_case: bool = False) -> Any:
         """
         Override of get() that takes globals and compound keys into account
         :param key: dict key of item to get
         :param default: default to return if key is not found
+        :param no_case: whether to ignore case when searching for key
         :return: self[key]
         """
+        if no_case and isinstance(key, str):
+            for k in self.keys():
+                if k.lower() == key.lower():
+                    key = k
+                    break
         if key not in self:
             return default
         try:
@@ -423,7 +442,7 @@ class DictConfig(dict):
                 new_globals = value.globals.copy()
                 new_globals.update(self.globals)
                 # update the main globals to match
-                if self.globals is not None:
+                if self.globals:
                     self.globals.update(new_globals)
                 else:
                     self.globals = new_globals
@@ -460,7 +479,7 @@ class DictConfig(dict):
         # recurse into the copy, replacing DictConfig with dict
         self._configs_to_dict(result, skip_iterables=skip_iterables or skip_lists)
         if with_globals:
-            result[GLOBALS_KEY] = dict(self.globals) if self.globals is not None else {}
+            result[GLOBALS_KEY] = dict(self.globals)
         return result
 
     def copy(self) -> 'DictConfig':
@@ -473,9 +492,9 @@ class DictConfig(dict):
     def update(self, other: Mapping = None, **kwargs) -> 'DictConfig':
         if other is not None:
             if hasattr(other, 'globals'):
-                if self.globals is not None:
+                if self.globals:
                     self.globals.update(other.globals)
-                elif other.globals is not None:
+                elif other.globals:
                     self.globals = other.globals.copy()
             for k, v in other.items():
                 if k in self and isinstance(self[k], DictConfig):
@@ -560,7 +579,7 @@ class DictConfig(dict):
             for x in item:
                 node.append(cls._cfg2xml(x, '_'+tag, etree))
         elif isinstance(item, dict):
-            if cfg_globals is not None:
+            if cfg_globals:
                 node.append(cls._cfg2xml(cfg_globals, '_globals', etree))
             if exclude is None:
                 exclude = []
@@ -687,7 +706,7 @@ class DictConfig(dict):
                 if open_file:
                     source.close()
 
-        cfg.filename = filename
+        cfg.file_path = None if filename is None else Path(filename)
 
         return cfg
 
@@ -862,21 +881,19 @@ class DictConfig(dict):
                         v = var_name[(len(self.env_var_prefix)):]
                         if v[0] == '{' and v[-1] == '}':
                             environment[('_globals', v[1:-1])] = getenv(var_name)
-                            if self.globals is None:
-                                self.globals = {}
+                            if self.globals:
+                                self.globals = self.__class__({})
                             self.globals[v[1:-1]] = None
                         elif v in recursive_keys:
                             environment[recursive_keys[v]] = getenv(var_name)
                         else:
                             environment[v] = getenv(var_name)
                             self[v] = None
-            elif self.globals is not None:
+            elif self.globals:
                 for key in self.globals:
                     if (getenv('{{{}}}'.format(key)) is not None and
                             self._case_safe(key, exclude_vars) not in exclude_vars):
                         environment[('_globals', key)] = getenv('{{{}}}'.format(key))
-                        if self.globals is None:
-                            self.globals = {}
         else:
             # check if the given keys are in the environment (and not excluded)
             for key in env_vars:
@@ -884,8 +901,8 @@ class DictConfig(dict):
                     if (getenv(self.env_var_prefix + key) is not None and
                             self._case_safe(key, exclude_vars) not in exclude_vars):
                         environment[('_globals', key)] = getenv(self.env_var_prefix + key)
-                        if self.globals is None:
-                            self.globals = {}
+                        if not self.globals and not isinstance(self.globals, self.__class__):
+                            self.globals = self.__class__({})
                 else:
                     key = self._case_safe(key, recursive_keys)
                     if (getenv(self.env_var_prefix + key) is not None and
@@ -900,6 +917,7 @@ class DictConfig(dict):
         # perform update with constructed environment (not using compound keys, for simplicity)
         for keys, value in environment.items():
             if keys[0] == '_globals':
+                f = self.set_global
                 d = self.globals
                 if keys[-1] not in d:
                     continue
@@ -907,16 +925,17 @@ class DictConfig(dict):
                 d = self
                 for key in keys[:-1]:
                     d = d[key]
+                f = d.__setitem__
             key = keys[-1]
 
             # for bool, check specific non-True values
             if isinstance(d[key], bool):
-                d[keys[-1]] = value.lower() not in ['0', 'false']
+                f(keys[-1], value.lower() not in ['0', 'false'])
             else:
                 # for other types, cast to type of existing key, or str if None
                 t = str if d[key] is None else type(d[key])
                 try:
-                    d[keys[-1]] = t(value)
+                    f(keys[-1], t(value))
                 except ValueError:
                     raise SyntaxError('Cannot cast {} to {} from environment'.format(value, t))
 
@@ -954,7 +973,7 @@ class DictConfig(dict):
             self[k] = v
         return excess
 
-    def _set_globals_from_args(self, k, v):
+    def _set_globals_from_args(self, k: str, v: Any):
         """
         helper function used by .update_from_arguments()
         :param k: key for value to set
@@ -962,11 +981,11 @@ class DictConfig(dict):
         :return: None
         """
         excess = []
-        if self.globals is None:
-            self.globals = {}
+        if not self.globals and not isinstance(self.globals, self.__class__):
+            self.globals = self.__class__({})
         # for bool, check specific non-True values
         if k in self.globals and isinstance(self.globals[k], bool):
-            self.globals[k] = v if isinstance(v, bool) else v.lower() not in ['0', 'false']
+            self.set_global(k, v if isinstance(v, bool) else v.lower() not in ['0', 'false'])
         else:
             # for other types, cast to type of existing key and remove excess values for non-iterable predefined values
             if k in self.globals:
@@ -979,11 +998,17 @@ class DictConfig(dict):
             else:
                 t = type(v)
             try:
-                self.globals[k] = t(v)
+                self.set_global(k, t(v))
             except ValueError:
                 raise SyntaxError('Cannot cast {} to {} from arguments'.format(v, t))
         return excess
 
+    def set_global(self, key: str, value: Any):
+        self.globals[key] = value
+        for k, v in self.items():
+            if isinstance(v, DictConfig) and k != '_globals':
+                v.set_global(key, value)
+
     def update_from_arguments(self, cli_args: Union[Dict[str, list], list] = None, aliases: Dict[str, str] = None):
         """
         Update the Config with values parsed from the command line arguments. Overwriting values will be cast to the
@@ -1101,6 +1126,8 @@ class Config(DictConfig):
 
     def __setattr__(self, attr, value):
         # check if configuration items can shadow object attributes, not using hasattr, as it will call __getattr__
+        # if attr in dir(self), it's already an attribute - set if keys can't shadow attributes
+        # if attr not in self, it's not an attribute and not an existing key - set if keys can't shadow attributes
         if (attr in dir(self) or attr not in self) and (not hasattr(self, '_shadow_attrs') or not self._shadow_attrs):
             super(DictConfig, self).__setattr__(attr, value)
         else:
