@@ -1,5 +1,6 @@
 import unittest
 from conffu import DictConfig, Config
+from conffu._config import argv_to_dict
 
 
 class TestConfig(unittest.TestCase):
@@ -133,6 +134,14 @@ class TestConfig(unittest.TestCase):
         cfg['b'] = 2
         self.assertEqual((2, 2, 2, 2), (cfg.a, cfg['a'], cfg.b, cfg['b']), msg='attributes can be shadowed')
 
+        cfg = Config()
+        cfg.shadow_attrs = False
+        cfg['a'] = 1
+        cfg.a = 2  # this still sets cfg['a']
+        cfg.b = 1
+        cfg['b'] = 2  # but this does not set cfg.b as it was set before
+        self.assertEqual((2, 2, 1, 2), (cfg.a, cfg['a'], cfg.b, cfg['b']), msg='unshadowed attributes retain value')
+
     def test_key_error(self):
         cfg = DictConfig({'test': 1})
         with self.assertRaises(KeyError, msg='without no_key_error, reading non-existent keys raises an exception'):
@@ -151,7 +160,6 @@ class TestConfig(unittest.TestCase):
         except KeyError:
             self.fail('get never raises a KeyError')
 
-
     def test_split_keys(self):
         cfg = Config({'test': {'nested': 1}})
         self.assertEqual(1, cfg['test.nested'], 'compound keys work as index')
@@ -180,6 +188,14 @@ class TestConfig(unittest.TestCase):
                          cfg.recursive_keys(),
                          'compound keys are generated in order, depth-first')
 
+    def test_compound_keys_globals(self):
+        cfg = Config({'_globals': {'x': 'foo'}, 'value': '{x} bar', 'ns': {'n': '{x}'}, 'ns2.n': '{x}'},
+                     no_compound_keys=True)
+        self.assertEqual('foo bar', cfg.value, 'globals work with compound keys disabled')
+        self.assertEqual('foo', cfg.ns.n, 'globals work with compound keys disabled, on nested elements')
+        with self.assertRaises(KeyError, msg='compound key will raise key error for nested when disabled'):
+            _ = cfg['ns.n']
+
     def test_copy(self):
         cfg = Config({'1': 2})
         cfg_copy = cfg.copy()
@@ -276,6 +292,24 @@ class TestConfig(unittest.TestCase):
             self.assertEqual('test', dcfg.xs[1], 'correct value added while not using globals through direct')
         self.assertTrue(cfg.disable_globals, 'still using globals outside context')
 
+    def test_inherit_changed_globals(self):
+        args = argv_to_dict(['script.py', '-{root}', 'foo', '-p', '{root}/bar'])
+        cfg = Config({"_globals": {"root": "baz"}, "sub": {"x": "{root}/qux"}}).update_from_arguments(args)
+        self.assertEqual(cfg['p'], 'foo/bar', msg='argument without parameters should be True in Config')
+        self.assertEqual(cfg['sub']['x'], 'foo/qux', msg='argument without parameters should be True in Config')
+
+    def test_set_global_propagation(self):
+        args = argv_to_dict(['script.py', '-{root}', 'foo', '-p', '{root}/bar'])
+        cfg = Config({"_globals": {"root": "baz"}, "sub": {"x": "{root}/qux"}}).update_from_arguments(args)
+        cfg.set_global('root', 'quux')
+        self.assertEqual(cfg['p'], 'quux/bar', msg='updated global works')
+        self.assertEqual(cfg['sub']['x'], 'quux/qux', msg='updated global propagated')
+
+    def test_shadow_attribute_initial_dict(self):
+        cfg = Config({'a': {'parameters': {'b': 1}}})  # parameters would conflict with self.parameters
+        self.assertEqual(1, cfg['a.parameters.b'], 'as key, content is preferred')
+        self.assertEqual(None, cfg.a.parameters, 'as attribute, attribute is preferred')
+
     def test_get(self):
         cfg = Config({'_globals':{'g': 2}, 'a': 1, 'b': '{g}', 'c': {'d': 3}})
         self.assertEqual(1, cfg.get('a'), 'get works as expected')
@@ -379,6 +413,15 @@ class TestConfig(unittest.TestCase):
         self.assertEqual('4', cfg.c, 'added global for update with globals')
         self.assertEqual('134', cfg.d, 'correct global substitutions for newly added keys after update with globals')
 
+    def test_no_case_keys(self):
+        cfg = Config({'a': 1, 'B': 2, 'c': 3, 'C': 4})
+        self.assertEqual((1, 2), (cfg.get('a', None), cfg.get('B', None)),
+                         'case-sensitive keys work as expected')
+        self.assertEqual((None, None), (cfg.get('A'), cfg.get('b')),
+                         'case-sensitive keys work as expected when missed')
+        self.assertEqual((1, 2), (cfg.get('A', None, no_case=True), cfg.get('b', None, no_case=True)),
+                         'non-case-sensitive keys work as expected')
+
 
 if __name__ == '__main__':
     unittest.main()
